package de.ibh.RandomArt;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

public class RAImage {
	private static int dheight = 48;
	private static int dwidth = 48;
	private static Rectangle dbounds = new Rectangle(0, 0, dwidth, dheight);
	
	private static AbstractRAOperator generate(final RAPRNG prng, List<AbstractRAOperator> ops, final int k) {
		if(k <= 0) {
			AbstractRAOperator op = (AbstractRAOperator) ((AbstractRAOperator) prng.choice(RAUtilities.filterOperatorsByArity(ops, 0, 0))).clone();
			op.init(null);

			return op;
		}
		AbstractRAOperator op = (AbstractRAOperator) ((AbstractRAOperator) prng.choice(RAUtilities.filterOperatorsByArity(ops, 1, null))).clone();

		List<AbstractRAOperator> args = new LinkedList<AbstractRAOperator>();
		for(int i = 0; i < op.getArity(); i++) {
			args.add(generate(prng, ops, k-1));
		}
		
		op.init(args);
		return op;
	}
	
	public static void setDefaultBounds(int width, int height) {
		if(RAImage.dwidth != width || RAImage.dheight != height) {
			RAImage.dwidth = width;
			RAImage.dheight = height;
			
			dbounds = new Rectangle(0, 0, width, height);
		}
	}
	
	public static int getDefaultWidth() {
		return dwidth;
	}
	
	public static int getDefaultHeight() {
		return dheight;
	}
	
	public static Image create(String seed, final Device dev) {
		return create(seed, dev, dbounds);
	}
	
	public static Image create(long seed, final Device dev) {
		return create(seed, dev, dbounds);
	}
	
	public static Image create(String seed, final Device dev, final int width, final int height) {
		return create(seed, dev, new Rectangle(0, 0, width, height));
	}
	
	public static Image create(long seed, final Device dev, final int width, final int height) {
		return create(seed, dev, new Rectangle(0, 0, width, height));
	}
	
	protected static Image create(String seed, final Device dev, final Rectangle bounds) {
		if(seed != null)
			return create(getSeed(seed), dev, bounds);

		Image img = new Image(dev, bounds);
		GC gc = new GC(img);

		gc.setBackground(dev.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		gc.fillRectangle(bounds);
		
		return img;
	}
	
	protected static Image create(long seed, final Device dev, final Rectangle bounds) {
        Image img = new Image(dev, bounds);
		GC gc = new GC(img);

		AbstractRAOperator exprTree = getExprTree(seed);
		
		for(int x = 0; x < bounds.width; x++) {
			for(int y = 0; y < bounds.height; y++) {
				final Rectangle rect = new Rectangle(x, y, 1, 1);

				double rgb[] = exprTree.eval(2.0*(x + 1)/bounds.width - 1.0, 2.0*(y + 1)/bounds.height - 1.0);
				Color col = new Color(dev, new RGB(
					(int) (127.5*(1 + rgb[0])),
					(int) (127.5*(1 + rgb[1])),
					(int) (127.5*(1 + rgb[2]))
				));
				gc.setBackground(col);
				gc.fillRectangle(rect);
			}
		}
		
		return img;
	}
	
	public static long getSeed(final String seed) {
		return seed.hashCode();
	}
	
	protected static AbstractRAOperator getExprTree(long seed) {
		RAPRNG prng = new RAPRNG(seed);

		return generate(prng, RAUtilities.getOperators(prng), 7);
	}
	
	public static String printExprTree(String seed) {
		return printExprTree(getSeed(seed));
	}
	
	protected static String printExprTree(long seed) {
		AbstractRAOperator exprTree = getExprTree(seed);

		return printExpr(exprTree);
	}
	
	protected static String printExpr(AbstractRAOperator op) {
		String s = op.getClass().getSimpleName().replaceFirst("^RAO", "");
		s += "(";
		
		if(op.getOps() != null) {
			s += " ";

			for(Iterator<AbstractRAOperator> it = op.getOps().iterator(); it.hasNext(); ) {
				s += printExpr(it.next());
				if(it.hasNext())
					s+= ", ";
			}
			
			s += " ";
		}
		s += ")";
		
		return s;
	}
}
