package de.ibh.StrongRoom.Client.Config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Collections;
import java.util.InvalidPropertiesFormatException;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;

public abstract class AbstractConf {
    protected Properties prop = new Properties();
	protected static AbstractConf conf = null;

	public abstract String getConfName();
	
	public static String getDirName() {
    	final String sep =  System.getProperty("file.separator");
		
    	return System.getenv().get("SRHOME") != null ?
    	        System.getenv().get("SRHOME") + sep :
    	        System.getProperty("user.home") + sep  + ".StrongRoom" + sep;
	}
	
    public String getFileName() {
    	return getDirName() + getConfName() + ".xml";
    }
    
	public AbstractConf() throws InvalidPropertiesFormatException, IOException {
		final String fn = getFileName();
		InputStream xmlStream;
		
		if(fn.contains("://")) {
			URL u = new URL(fn);
			xmlStream = u.openStream();
		}
		else
			xmlStream = new FileInputStream(getFileName());
        
		try {
        	prop.loadFromXML(xmlStream);
        }
        finally {
            xmlStream.close();
        }
	}
    
	public AbstractConf(boolean silent) {
		try {
	        InputStream xmlStream = new FileInputStream(getFileName());
	        try {
	        	prop.loadFromXML(xmlStream);
	        }
	        finally {
	            xmlStream.close();
	        }
		}
		catch(Exception e) {
			/* Ignore any exceptions. */
		}
	}
    
	public void reload() throws InvalidPropertiesFormatException, IOException {
        InputStream xmlStream = new FileInputStream(getFileName());
        try {
        	prop.loadFromXML(xmlStream);
        }
        finally {
            xmlStream.close();
        }
	}
	
    public void saveConfig() throws IOException {
    	new File(getDirName()).mkdir();

    	OutputStream xmlStream = new FileOutputStream(getFileName());
        try {
        	prop.storeToXML(xmlStream, null);
        }
        finally {
        	xmlStream.close();
        }
    }
    
    public void saveConfigSilent() {
		try {
	    	new File(getDirName()).mkdir();

	    	OutputStream xmlStream = new FileOutputStream(getFileName());
	        try {
	        	prop.storeToXML(xmlStream, null);
	        }
	        finally {
	        	xmlStream.close();
	        }
		}
		catch(Exception e) {
			/* Ignore any exceptions. */
		}
    }

    public abstract ECryptoPGP_KeyType getSignKeyType();
	public abstract String getSignString();

	public static String buildSignString(List<String> l) {
		Collections.sort(l);
		
		String sign = "<|";
		
		for(Iterator<String> i = l.iterator(); i.hasNext(); ) {
			sign += i.next() + "|";
		}
		
		sign += ">";

		return sign.replaceAll("\r", "");
	}
}
