package de.ibh.StrongRoom.Client.Lucene;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.document.Field.TermVector;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.util.Version;

import de.ibh.StrongRoom.Client.Crypto;
import de.ibh.StrongRoom.Client.Misc;
import de.ibh.StrongRoom.Client.SWT.PassphraseDialog;
import de.ibh.StrongRoom.Crypto.IRunnable;
import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;
import de.ibh.StrongRoom.DB.CryptoData;
import de.ibh.StrongRoom.DB.CryptoEntry;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.Role;
import de.ibh.StrongRoom.Exception.SRGenericCryptoEngineException;
import de.ibh.StrongRoom.Exception.SRGenericException;
import de.ibh.StrongRoom.Exception.SRSecretKeyNotFoundException;

public class SRSearch {
    public enum EDocFields {
        _CE,

        DATE,
        SUBJECT,
        DESCRIPTION,
        ROLE,
        CUSTOMER,
        CONTENT,
    };

    static final QueryParser qp = new QueryParser(Version.LUCENE_31, EDocFields.CONTENT.name(), new StandardAnalyzer(Version.LUCENE_31));

	static private final HashMap<Long, Directory> srdirs = new HashMap<Long, Directory>();

	public static void buildIndex(final ISRIndexUpdateProgress up, final Role role, final DBConn dbconn, final Crypto crypto, final IRunnable ira, final Object userData) throws SRGenericException, CorruptIndexException, IOException {
		/*final SRDirectory srdir = new SRDirectory(dbconn, crypto, role, new IRetrievePublicKeyRingHelper() {
			@Override
			public String getPublicKeyring(ECryptoPGP_KeyType kt, Object userData) {
				if(kt != ECryptoPGP_KeyType.ROLE_KEY)
					return null;

				return role.getRoleKey().getPublicKey();
			}
		}, null, userData);
		srdir.clearIndex();*/
		
		final Directory srdir = new RAMDirectory();
		
		final IndexWriterConfig idxwrrcfg = new IndexWriterConfig(Version.LUCENE_31, new StandardAnalyzer(Version.LUCENE_31));
		final IndexWriter idxwr = new IndexWriter(srdir, idxwrrcfg);

		String roleKey = Misc.b2s(crypto.decrypt(ECryptoPGP_KeyType.USER_KEY, role.getRoleKey().getEncryptedData().getBytes(), PassphraseDialog.get(), ira, userData));
		try {
			final List<CryptoEntry> ces = dbconn.getCryptoEntriesByRole(role);
			up.init("Indexing '" + role.getName() + "'...", ces.size());
			int idx = 0;
			for(CryptoEntry ce : ces) {
				final List<CryptoData> cds = dbconn.getCryptoData(ce);
				
				for(CryptoData cd : cds) {
					final Document doc = new Document();
					doc.add(new Field(EDocFields._CE.name(), Long.toString(ce.getId()), Store.YES, Index.NO));
					doc.add(new Field(EDocFields.DATE.name(), cd.getDate().toString(), Store.NO, Index.NOT_ANALYZED));
					doc.add(new Field(EDocFields.SUBJECT.name(), cd.getCE().getSubject(), Store.NO, Index.ANALYZED));
					doc.add(new Field(EDocFields.DESCRIPTION.name(), cd.getCE().getDescription(), Store.NO, Index.ANALYZED));
					doc.add(new Field(EDocFields.ROLE.name(), cd.getCE().getRole().getName(), Store.NO, Index.NOT_ANALYZED));
					doc.add(new Field(EDocFields.CUSTOMER.name(), cd.getCE().getCustomer().getName(), Store.NO, Index.NOT_ANALYZED));
					doc.add(new Field(EDocFields.CONTENT.name(), Misc.b2s(crypto.decrypt(roleKey, cd.getEncryptedData().getBytes(), userData)), Store.NO, Index.ANALYZED, TermVector.YES));

					idxwr.addDocument(doc);
				}
				
				idx++;

				up.progress(idx);
			}
		
			idxwr.close();
			
			srdirs.put((Long) role.getID(), srdir);
		}
		finally {
			roleKey = null;
			System.gc();
		}
	}
	
	public static List<SearchResult> searchIndex(final Role role, final DBConn dbconn, final Crypto crypto, final Object userData, final String queryString, final int n) throws SRSecretKeyNotFoundException, SRGenericCryptoEngineException, IOException, ParseException {
		final Directory srdir = srdirs.get(role.getID());
		
		if(srdir == null)
			return null;
		
		final Query query = qp.parse(queryString);
		final IndexSearcher searcher = new IndexSearcher(srdir);
		try {
			final TopDocs tds = searcher.search(query, n);
			final List<SearchResult> srl = new LinkedList<SearchResult>();
			for(ScoreDoc sd : tds.scoreDocs) {
				srl.add(new SearchResult(role, searcher.doc(sd.doc), sd.score));
			}
		
			return srl;
		}
		finally {
			searcher.close();
		}
	}
}
