/**
 * 
 */
package de.ibh.StrongRoom.Client.SWT;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.InvalidPropertiesFormatException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.BasicConfigurator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.ibh.StrongRoom.Client.Crypto;
import de.ibh.StrongRoom.Client.DB;
import de.ibh.StrongRoom.Client.Misc;
import de.ibh.StrongRoom.Client.Config.BaseConf;
import de.ibh.StrongRoom.Client.Config.SigningConf;
import de.ibh.StrongRoom.Crypto.IRetrievePublicKeyRingHelper;
import de.ibh.StrongRoom.Crypto.IRetrieveSecretKeyRingHelper;
import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;
import de.ibh.StrongRoom.DB.CryptoData;
import de.ibh.StrongRoom.DB.CryptoEntry;
import de.ibh.StrongRoom.DB.Customer;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.Role;
import de.ibh.StrongRoom.DB.UserRole;
import de.ibh.StrongRoom.Exception.SRDBSQLException;
import de.ibh.StrongRoom.Exception.SRGenericCryptoEngineException;
import de.ibh.StrongRoom.Exception.SRGenericException;
import de.ibh.StrongRoom.Exception.SRPublicKeyNotFoundException;
import de.ibh.StrongRoom.Exception.SRRoleKeyMismatchException;
import de.ibh.StrongRoom.Exception.SRRoleNotFoundException;
import de.ibh.StrongRoom.Exception.SRSecretKeyNotFoundException;
import de.ibh.StrongRoom.Exception.SRSigningKeyNotConfirmedException;
import de.ibh.StrongRoom.Log.DebugMessage;
import de.ibh.StrongRoom.Log.ErrorMessage.EMessageTypes;
import de.ibh.StrongRoom.l10n.StrongRoomL10N;

/**
 * @author ellguth
 *
 */
public class ImportData {
    private Shell sShell;
    private Crypto crypto = null;
    private Set<ParseData> parseData;
    private ErrorMsgDialog errmsg = null;    
    private DBConn   db = null;
    private BaseConf baseconf   = null;
    private SigningConf signingconf = null;
    private Combo cmbRoles = null;
    private final Map<String, Role> roleMap = new HashMap<String, Role>();
    private final StrongRoomL10N  l10n = StrongRoomL10N.getInstance();
    
    private DebugMessage logger = DebugMessage.getInstance();
    
    private class ParseData {
        private String  id      = null;
        private String  subject = null;
        private String  descr   = null;
        
        private HashMap<Timestamp, String> cdMap = null;
        
        protected ParseData() { }

        protected boolean validate() {
            if(id == null) return false;
            if(subject == null) return false;
            try {
                Integer.parseInt(id);
            } catch (Exception e){
                return false;
            }
            if(cdMap != null && cdMap.size() > 0) {
                for(Iterator<Timestamp> it = cdMap.keySet().iterator(); it.hasNext();) {
                	Timestamp tstamp = it.next();

                    
                    if(cdMap.get(tstamp) == null)
                        return false;
                }
            }
            
            return true;
        }
        
        protected void setId(String id) {
            this.id = id;
        }
        
        protected void setSubject(String subject) {
            this.subject = subject;
        }
        
        protected void setDescr(String descr) {
            this.descr = descr;
        }
        
        protected void setCryptoData(HashMap<Timestamp, String> cdMap) {
            this.cdMap = cdMap;
        }
        
        protected String getId() {
            return id;
        }
        
        protected String getSubject() {
            return subject;
        }
        
        protected String getDescr() {
            return descr;
        }
        
        protected HashMap<Timestamp, String> getCryptoData() {
            return cdMap;
        }
    }

    public static void main(String[] args) {
		// Setup logging
	    BasicConfigurator.configure();
	    
		new ImportData();
		System.exit(0);
	}
    
    public ImportData() {
        Display display = Display.getDefault();
        
        logger.debugEnable(true);
        createSShell();
        sShell.open();
        
        dbinit();
    
        while (!sShell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
    }
    
    
    @SuppressWarnings("deprecation")
	private void dbinit() {
        try {
            crypto = new Crypto(baseconf, true, SilentPassphraseDialog.get());
            
            db = DB.getDBConn(crypto, baseconf, signingconf, new InputDialog(sShell), PassphraseDialog.get(), SigningKeyVerificationDialog.get(), sShell, errmsg);
        } catch (SRGenericException e) {
            crypto = null;
            errmsg.printErrorMessage(e.getMessage(), EMessageTypes.TYPE_ERROR);
            
            sShell.dispose();
            System.exit(0);
        }
        
        if(db == null) {
            errmsg.printErrorMessage(l10n._("Can't get crypto instance."), EMessageTypes.TYPE_ERROR);
            
            sShell.dispose();
            System.exit(0);            
        }        
        
        fillRoleList();
    }
     
    
    private void fillRoleList() {
        Map<Long, Role> roles;
        try {
            roles = db.getRoles();
        } catch (SRRoleNotFoundException e) {
            logger.debug(e);

            return;
        }
        
        for(Iterator<UserRole> it = db.getUserRolesByFPR(crypto.getFPR(ECryptoPGP_KeyType.USER_KEY)).iterator(); it.hasNext(); ) {
            Role role = roles.get(it.next().getRoleID());
            roleMap.put(role.getName(), role);
            cmbRoles.add(role.getName());
        }
        
        if(cmbRoles.getItemCount() == 0) {
            errmsg.printErrorMessage(l10n._("You don't have any role to select."), EMessageTypes.TYPE_WARN);
            return;
        }        
    }
    
    
    private Set<ParseData> parseXML(final String path) {
        File f = new File(path);
        if(!f.exists())
            return null;
        
        Set<ParseData> parseData = new HashSet<ParseData>();
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(f);
            doc.normalize();
            
            if(!doc.getDocumentElement().getNodeName().equals("srdata"))
                // TODO print errmsg.
                return null;
            
            NodeList nlce = doc.getElementsByTagName("ce");
            
            for(int i=0; i<nlce.getLength();i++) {
                Node nce = nlce.item(i);
                NodeList nl = nce.getChildNodes();
                ParseData pd = new ParseData();

                for(int j=0; j<nl.getLength();j++) {
                    Node n = nl.item(j);
                    if(n.getNodeName().equals("id")) {
                        pd.setId(n.getTextContent());
                    } else if(n.getNodeName().equals("subject")) {
                        pd.setSubject(n.getTextContent());
                    } else if(n.getNodeName().equals("descr")) {
                        pd.setDescr(n.getTextContent());
                    } else if(n.getNodeName().equals("cd")) {
                        NodeList nlcd = n.getChildNodes();
                        HashMap<Timestamp, String> cdMap = new HashMap<Timestamp, String>();
                        Timestamp tstamp = null;
                        String data = null;

                        for(int k=0; k<nlcd.getLength();k++) {
                            Node ncd = nlcd.item(k);

                            if(ncd.getNodeName().equals("tstamp")) {
                                tstamp = new Timestamp(new Date().getTime());
                            } else if (ncd.getNodeName().equals("data")) {
                                data = ncd.getTextContent();
                            }
                                
                        }
                            
                        if(tstamp != null && data != null)
                            cdMap.put(tstamp, data);
                        
                        pd.setCryptoData(cdMap);
                    }
                }
                
                parseData.add(pd);
            }
        } catch (ParserConfigurationException e) {
            logger.debug(e);
        } catch (SAXException e) {
            logger.debug(e);
        } catch (IOException e) {
            logger.debug(e);
        }
        
        return parseData;
    }
    
    
    /**
     * This method initializes sShell
     */
    private void createSShell() {
        sShell = new Shell();

        sShell.setText(l10n._("StrongRoom Data Import"));
        sShell.setLayout(new GridLayout(3, false));
        
        errmsg = ErrorMsgDialog.getInstance(sShell);
        
        try {
            baseconf = BaseConf.getInstance();
        } catch (InvalidPropertiesFormatException e) {
             errmsg.printErrorMessage(l10n._("Could not parse base configuration: ") + e.getMessage(), EMessageTypes.TYPE_ERROR);
             sShell.dispose();
             System.exit(0);
        } catch (IOException e) {
             errmsg.printErrorMessage(l10n._("Could not read base configuration: ") + e.getMessage(), EMessageTypes.TYPE_ERROR);
             sShell.dispose();
             System.exit(0);
        }
        
        try {
            signingconf = SigningConf.getInstance();
        } catch (InvalidPropertiesFormatException e) {
             errmsg.printErrorMessage(l10n._("Could not parse signing configuration: ") + e.getMessage(), EMessageTypes.TYPE_WARN);
             signingconf = SigningConf.getInstanceSilent();
        } catch (IOException e) {
             errmsg.printErrorMessage(l10n._("Could not read signing configuration: ") + e.getMessage(), EMessageTypes.TYPE_WARN);
             signingconf = SigningConf.getInstanceSilent();
        }
        
        sShell.addListener(SWT.Close, new Listener() {
            
            @Override
            public void handleEvent(Event event) {
                sShell.dispose();
                event.doit = true;
                System.exit(0);
            }
        });        
        
        final ScrolledComposite sc = new ScrolledComposite(sShell, SWT.V_SCROLL);       
        sc.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, true));
        
        Composite c = new Composite(sc, SWT.NONE);
        c.setLayout(new GridLayout(2, false));
        c.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, true));

        // Select file
        Group g = new Group(c, SWT.SHADOW_ETCHED_IN);
        g.setText(l10n._("Import File"));
        g.setLayout(new GridLayout(3, false));
        g.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        
        Label l = new Label(g, SWT.NONE);
        l.setText(l10n._("Select file to import:"));
        l.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        
        final Text pathText = new Text(g, SWT.BORDER);
        pathText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        
        final Button selectFileButton = new Button(g, SWT.BORDER);
        selectFileButton.setText(l10n._("Select File..."));
        selectFileButton.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        selectFileButton.addListener(SWT.Selection, new Listener() {
            
            @Override
            public void handleEvent(Event event) {
                final FileDialog fileDialog = new FileDialog(sShell);
                
                String path = fileDialog.open();
                if(path != null) pathText.setText(path);
            }
        });
        
        // Options
        g = new Group(c, SWT.SHADOW_ETCHED_IN);
        g.setText(l10n._("Options"));
        g.setLayout(new GridLayout(3, false));
        g.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        
        l = new Label(g, SWT.NONE);
        l.setText(l10n._("Select a role:"));
        l.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false, 1, 1));
        
        cmbRoles = new Combo(g, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
        cmbRoles.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        
        final Button roButton = new Button(g, SWT.CHECK);
        roButton.setText(l10n._("Readonly"));
        roButton.setSelection(true);
        roButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
        
        final Button exportableButton = new Button(g, SWT.CHECK);
        exportableButton.setText(l10n._("Exportable"));
        exportableButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
        
        final Button hiddenButton = new Button(g, SWT.CHECK);
        hiddenButton.setText(l10n._("Hidden"));
        hiddenButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));        
                
        // Import status
        g = new Group(c, SWT.SHADOW_ETCHED_IN);
        g.setText(l10n._("Import Status"));
        g.setLayout(new GridLayout(2, false));
        g.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        
        l = new Label(g, SWT.NONE);
        l.setText(l10n._("Valid records found:"));
        l.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        
        final Label validRecords = new Label(g, SWT.NONE);
        validRecords.setText("0");
        validRecords.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        
        l = new Label(g, SWT.NONE);
        l.setText(l10n._("Import progress:"));
        l.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        
        final ProgressBar progressBar = new ProgressBar(g, SWT.NULL); 
        progressBar.setMaximum(100);
        progressBar.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        progressBar.setEnabled(false);
        
        // Buttons
        final Button parseButton = new Button(c, SWT.BORDER);
        final Button importButton = new Button(c, SWT.BORDER);
        parseButton.setText(l10n._("Parse File"));
        parseButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
        
        parseButton.addListener(SWT.Selection, new Listener() {
            
            @Override
            public void handleEvent(Event event) {
                parseData = parseXML(pathText.getText());
                
                int valid = 0;
                for(Iterator<ParseData> it = parseData.iterator(); it.hasNext();) {
                    ParseData pd = it.next();
                    if(pd.validate()) valid++;
                }
                if(valid > 0) {
                    importButton.setEnabled(true);
                    progressBar.setEnabled(true);
                    progressBar.setMaximum(valid);
                }

                validRecords.setText(Integer.toString(valid));
                validRecords.pack();
                validRecords.redraw();
                
            }
        });

        importButton.setText(l10n._("Import Data"));
        importButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
        importButton.setEnabled(false);
        
        Listener importListener  = new Listener() {
            @Override
            public void handleEvent(Event event) {
                final Role role = roleMap.get(cmbRoles.getText());
                if(role == null) {
                    errmsg.printErrorMessage(l10n._("No role selected!"), EMessageTypes.TYPE_WARN);
                    return;
                }

                int progress = 0;                
                for(Iterator<ParseData> it = parseData.iterator(); it.hasNext();) {
                    ParseData pd = it.next();
                    if(pd.validate() != true) 
                        continue;

                    final Customer cu = new Customer (Integer.valueOf(pd.getId()), "");
                    String subject = pd.getSubject();
                    
                    try {
                        List<CryptoEntry> ces = db.getCryptoEntries(cu);
                        
                        for(int i = 0; ; i++) {
                            boolean f = false;
                            if(i>0)
                                subject = pd.getSubject() + " #" + i;

                            for(Iterator<CryptoEntry> itce = ces.iterator(); itce.hasNext();) {
                                CryptoEntry ce = itce.next();
                            
                                if(ce.getSubject().equals(subject)) {
                                    f = true;
                                    break;
                                }
                            }
                            
                            if(!f)
                                break;
                        }
                    } catch (SRRoleKeyMismatchException e) {
                        logger.debug(e);
                    } catch (SRRoleNotFoundException e) {
                        logger.debug(e);
                    }
                    
                    final CryptoEntry newce = new CryptoEntry(
                            -1,
                            cu, // customer
                            subject,
                            pd.getDescr(),
                            role,
                            role.getRoleKey().getKeyFPR(),
                            exportableButton.getSelection(),
                            roButton.getSelection(),
                            hiddenButton.getSelection(),
                            crypto.getFPR(ECryptoPGP_KeyType.USER_KEY),
                            null,
                            null);
                    
                    try {
                        crypto.signRow(newce, new IRetrieveSecretKeyRingHelper() {
                            
                            @Override
                            public String getSecretKeyring(ECryptoPGP_KeyType kt, Object userData) {
                                if(kt != ECryptoPGP_KeyType.ROLE_KEY)
                                    return null;

								try {
									return Misc.b2s(crypto.decrypt(ECryptoPGP_KeyType.USER_KEY, role.getRoleKey(), userData));
								} catch (SRGenericException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();

									return null;
								}
                            }
                        }, sShell);
                        
                        DBConn.getInstance().createCryptoEntry(newce);   
                    } catch (SRSigningKeyNotConfirmedException e) {
                        logger.debug(e);
                    } catch (SRSecretKeyNotFoundException e) {
                        logger.debug(e);
                    } catch (SRDBSQLException e) {
                        logger.debug(e);
					} catch (SRGenericCryptoEngineException e) {
                        logger.debug(e);
					}
                    
                    final CryptoEntry ce;
                    CryptoEntry _ce = null;
                    try {
                        List<CryptoEntry> ces = db.getCryptoEntries(cu);
                        
                        for(Iterator<CryptoEntry> itce = ces.iterator(); itce.hasNext();) {
                            _ce = itce.next();
                            
                            if(_ce.getSubject().equals(subject))
                                break;
                            else
                                _ce = null;
                        }
                    } catch (SRRoleKeyMismatchException e2) {
                        logger.debug(e2);
                    } catch (SRRoleNotFoundException e2) {
                        logger.debug(e2);
                    }
                    
                    ce = _ce;
                    
                    for(Iterator<Timestamp> itcd = pd.getCryptoData().keySet().iterator(); itcd.hasNext();) {
                        final Timestamp tstamp = itcd.next();
                        
                        final CryptoData cd = new CryptoData(
                                ce,
                                tstamp,
                                null,
                                crypto.getFPR(ECryptoPGP_KeyType.USER_KEY),
                                null,
                                null);
                        
                        try {
                        	final String ft = pd.getCryptoData().get(tstamp);
                            crypto.encryptRow(cd, ft, new IRetrievePublicKeyRingHelper() {
                                
                                @Override
                                public String getPublicKeyring(ECryptoPGP_KeyType kt, Object userData) {
                                    if(kt != ECryptoPGP_KeyType.ROLE_KEY)
                                        return null;
      
                                    return ce.getRole().getRoleKey().getPublicKey();
                                }
                            }, sShell);

                            crypto.signRow(cd, new IRetrieveSecretKeyRingHelper() {
                                
                                @Override
                                public String getSecretKeyring(ECryptoPGP_KeyType kt, Object userData) {
                                    if(kt != ECryptoPGP_KeyType.ROLE_KEY)
                                        return null;
      
                                    try {
										return Misc.b2s(crypto.decrypt(ECryptoPGP_KeyType.USER_KEY, ce.getRole().getRoleKey(), userData));
									} catch (SRGenericException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();

										return null;
									}
                                }
                            }, sShell);
                            
                            db.saveCryptoData("1970-01-01 00:00:00", ce, cd);
                        } catch (SRSigningKeyNotConfirmedException e) {
                            logger.debug(e);
                        } catch (SRSecretKeyNotFoundException e) {
                            logger.debug(e);
                        } catch (SRPublicKeyNotFoundException e) {
                            logger.debug(e);
                        } catch (SRDBSQLException e) {
                            logger.debug(e);
                        } catch (SRGenericCryptoEngineException e) {
                            logger.debug(e);
						}
                    }
                    
                    progressBar.setSelection(++progress);
                }
                
                errmsg.printErrorMessage(l10n._("Import done. ") + (progress == progressBar.getMaximum() ? l10n._("Without errors.") : l10n._("With errors.")), progress == progressBar.getMaximum() ? EMessageTypes.TYPE_INFO : EMessageTypes.TYPE_WARN);
                parseData = null;
                progressBar.setEnabled(false);
                progressBar.setSelection(0);
                validRecords.setText("0");
                importButton.setEnabled(false);
            }
        };
        importButton.addListener(SWT.Selection, importListener);        
        
        sc.setContent(c);
        sc.setExpandHorizontal(true);
        sc.setExpandVertical(true);
        sc.setMinSize(c.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        
        sShell.pack();
        sShell.setSize(sShell.getSize().x*2, sShell.getSize().y);
    }    
}
