/**
 * 
 */
package de.ibh.StrongRoom.Client.SWT;

import java.util.Iterator;
import java.util.List;

import javax.management.relation.RoleNotFoundException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.ibh.StrongRoom.Client.Misc;
import de.ibh.StrongRoom.Crypto.IRetrieveSecretKeyRingHelper;
import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;
import de.ibh.StrongRoom.DB.CryptoEntry;
import de.ibh.StrongRoom.DB.Customer;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.UserRole;
import de.ibh.StrongRoom.Exception.SRDBSQLException;
import de.ibh.StrongRoom.Exception.SRGenericCryptoEngineException;
import de.ibh.StrongRoom.Exception.SRGenericException;
import de.ibh.StrongRoom.Exception.SRRoleKeyMismatchException;
import de.ibh.StrongRoom.Exception.SRRoleNotFoundException;
import de.ibh.StrongRoom.Exception.SRSecretKeyNotFoundException;
import de.ibh.StrongRoom.Exception.SRSigningKeyNotConfirmedException;
import de.ibh.StrongRoom.Log.DebugMessage;
import de.ibh.StrongRoom.Log.ErrorMessage;
import de.ibh.StrongRoom.Log.ErrorMessage.EMessageTypes;
import de.ibh.StrongRoom.l10n.StrongRoomL10N;


/**
 * @author ellguth
 *
 */

class SearchItem {
    private Tree                  tree;
    private TabFolders            tfolders = null;
    private Image                 iconFolder     = null;
    private Image                 iconFolderOpen = null;
    private Image                 iconFolderGrey = null;
    private Listener              subTreeItemClickListener = null;
    private GUI                   gui;
    static  DebugMessage          logger = DebugMessage.getInstance();
    private final StrongRoomL10N  l10n = StrongRoomL10N.getInstance();
    private MainMenu              mainmenu = null;
    
    /**
     * @param parent
     */
    public SearchItem(final Composite parent, final GUI gui, final TabFolders tfolders, final ErrorMessage errmsg) {
        tree = new Tree(parent, SWT.FILL | SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL);
        tree.setHeaderVisible(false);
        
        this.gui = gui;

        mainmenu = MainMenu.getInstance(parent.getShell());
        mainmenu.addItemListener(MainMenu.ITEM_SHOWHIDDEN, new Listener() {
            
            @Override
            public void handleEvent(Event event) {
                refreshTree(DBConn.getInstance());
            }
        });
        
        iconFolder     = GUI.getIcon(tree.getDisplay(), "folder.png");
        iconFolderOpen = GUI.getIcon(tree.getDisplay(), "folder_open.png");
        iconFolderGrey = GUI.getIcon(tree.getDisplay(), "folder_grey.png");

        tfolders.setDefault(tree);
        
        tree.addListener(SWT.MenuDetect, new Listener() {
            public void handleEvent(Event event) {
                int iboundx = event.x - tree.getParent().toDisplay(tree.getBounds().x, tree.getBounds().y).x;
                int iboundy = event.y - tree.getParent().toDisplay(tree.getBounds().x, tree.getBounds().y).y;
                
                TreeItem ti = tree.getItem(new Point(iboundx, iboundy));
                
                if(ti != null) {
                    showTreeContextMenu(tree.getShell(), tree.getDisplay(), ti, errmsg, event);
                }
            }
        });
        
        final TabFolders tfs = tfolders;
        final GUI g = gui;
        tree.addKeyListener(new KeyListener() {
			@Override
			public void keyReleased(KeyEvent arg0) {
			}
			
			@Override
			public void keyPressed(KeyEvent arg0) {
				if(arg0.keyCode != SWT.CR)
					return;
				
                final TreeItem ti[] = tree.getSelection();
                if(ti.length == 1 && ti[0].getData() instanceof CryptoEntry)
                    tfs.addCryptoTab((CryptoEntry) ti[0].getData(), DBConn.getInstance(), g.getCrypto());
			}
		});
  
        this.tfolders = tfolders;
    }
    
    
    private boolean updateCryptoEntry(final String subject, final boolean readonly, final boolean hidden, final Shell shell, final TreeItem ti, final ErrorMessage errmsg)
    {
        if(!(ti.getData() instanceof CryptoEntry))
            return false;
            
        final CryptoEntry ce = new CryptoEntry((CryptoEntry) ti.getData());
        ce.setSubject(subject);
        ce.setReadonly(readonly);
        ce.setHidden(hidden);
        
        try {
            gui.getCrypto().signRow(ce, new IRetrieveSecretKeyRingHelper() {
                
                @Override
                public String getSecretKeyring(ECryptoPGP_KeyType kt, Object userData) {
                    if(kt != ECryptoPGP_KeyType.ROLE_KEY)
                        return null;

                    try {
                        return Misc.b2s(gui.getCrypto().decrypt(ECryptoPGP_KeyType.USER_KEY, ce.getRole().getRoleKey(), userData));
                    } catch (SRGenericException e) {
                        errmsg.printErrorMessage(l10n._("Decryption error: ") + e.getLocalizedMessage(), EMessageTypes.TYPE_ERROR);

                        return null;
                    }
                }
            }, shell);
            
            DBConn db = DBConn.getInstance();

            db.updateCryptoEntry(ce);

            CryptoEntry _ce = db.getCryptoEntry(ce.getCustomer(), ce.getId());
            ti.setData(_ce);
            ti.setText(_ce.getSubject());
            
        } catch (SRGenericCryptoEngineException e) {
            errmsg.printErrorMessage(l10n._("Failed to create signature: ") + e.getMessage(), EMessageTypes.TYPE_WARN);
            logger.debug(e);
            return false;
        } catch (SRSigningKeyNotConfirmedException e) {
            errmsg.printErrorMessage(l10n._("Signing key not confirmed!"), EMessageTypes.TYPE_WARN);
            logger.debug(e);
            return false;
        } catch (SRSecretKeyNotFoundException e) {
            logger.debug(e);
            return false;
        } catch (SRDBSQLException e) {
            logger.debug(e);
            return false;
        } catch (SRRoleKeyMismatchException e) {
            logger.debug(e);
            return false;
        } catch (SRRoleNotFoundException e) {
            logger.debug(e);
            return false;
        }
        
        return true;
    }
    
    
    private void showTreeContextMenu(final Shell shell, Display display, final TreeItem ti, final ErrorMessage errmsg, Event event)
    {
        Menu popup = new Menu(shell, SWT.POP_UP);
        if((ti.getData() instanceof CryptoEntry) == false) {
            MenuItem item = new MenuItem(popup, SWT.PUSH);
            item.setText(l10n._("New entry"));
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event event) {
                    if((ti.getData() instanceof Customer) == false) return; 

                    DBConn db = DBConn.getInstance();
                    tfolders.addCryptoTab((Customer) ti.getData(), db, gui.getCrypto());                    
                }
              });
        } else {
            final CryptoEntry ce = (CryptoEntry) ti.getData();
            
            MenuItem item = new MenuItem(popup, SWT.PUSH);
            item.setText(l10n._("Open"));
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event event) {
                    DBConn db = DBConn.getInstance();
                    tfolders.addCryptoTab(ce, db, gui.getCrypto());                    
                }
              });

            new MenuItem(popup, SWT.SEPARATOR);
            
            item = new MenuItem(popup, SWT.PUSH);
            item.setText(l10n._("&Rename"));
            item.setEnabled(!ce.isReadonly());
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event event) {
                    InputDialog dlg = new InputDialog(shell);
                    dlg.setText(l10n._("Rename entry"));
                    dlg.setInput(ti.getText());
                    final String subject = dlg.open(true);

                    if(subject != null && !subject.equals(ti.getText()) && subject.length() >= 5) {
                        if(!updateCryptoEntry(subject, ce.isReadonly(), ce.isHidden(), shell, ti, errmsg))
                            errmsg.printErrorMessage(l10n._("Failed to change the crypto entry."), EMessageTypes.TYPE_WARN);
                    }
                }
              });
            
            item = new MenuItem(popup, SWT.CHECK);
            item.setText(l10n._("Read only"));
            item.setSelection(ce.isReadonly());
            
            final boolean readonly = !item.getSelection();
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event event) {
                    if(!updateCryptoEntry(ce.getSubject(), readonly, ce.isHidden(), shell, ti, errmsg))
                        errmsg.printErrorMessage(l10n._("Failed to change the crypto entry."), EMessageTypes.TYPE_WARN);
                    else
                        refreshTree(DBConn.getInstance());
                }
            });
            
            item = new MenuItem(popup, SWT.CHECK);
            item.setText(l10n._("Hidden"));
            item.setSelection(ce.isHidden());
            item.setEnabled(!ce.isReadonly());
            
            final boolean hidden = !item.getSelection();
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event event) {
                    if(!updateCryptoEntry(ce.getSubject(), ce.isReadonly(), hidden, shell, ti, errmsg))
                        errmsg.printErrorMessage(l10n._("Failed to change the crypto entry."), EMessageTypes.TYPE_WARN);
                    else
                        refreshTree(DBConn.getInstance());
                }
            });            
            
            new MenuItem(popup, SWT.SEPARATOR);
            
            item = new MenuItem(popup, SWT.PUSH);
            item.setText(l10n._("New entry"));
            item.addListener(SWT.Selection, new Listener() {
                public void handleEvent(Event e) {
                    if((ti.getParentItem() != null && ti.getParentItem().getData() instanceof Customer) == false) return; 

                    tfolders.addCryptoTab((Customer) ti.getParentItem().getData(), DBConn.getInstance(), gui.getCrypto());                    
                }
              });

        }

        popup.setLocation(event.x, event.y);
        popup.setVisible(true);
        while (!popup.isDisposed() && popup.isVisible()) {
          if (!display.readAndDispatch())
            display.sleep();
        }
        popup.dispose();        
    }    
    
    
    private int getSelectedItem() {
        int r = 0;
        
        for (TreeItem ti : tree.getSelection())
            r = tree.indexOf(ti);
        
        return r;
    }
    
    
    public boolean selectItemByName(String name) {
        tree.deselectAll();
       
        for(TreeItem ti : tree.getItems()) {
            if(ti.getText().toLowerCase().indexOf(name.toLowerCase()) >= 0) {
                tree.setSelection(ti);
                return true;
            }
        }
        
        return false;
    }
    
    
    public boolean selectItemByNameReverse(String name) {
        tree.deselectAll();
       
        for(int i = tree.getItemCount()-1; i >= 0; i--) {
            TreeItem ti = tree.getItem(i);
            if(ti.getText().toLowerCase().indexOf(name.toLowerCase()) >= 0) {
                tree.setSelection(ti);
                return true;
            }
        }
        
        return false;
    }    
    
    
    public boolean selectNextItemByName(String name) {
        int start = getSelectedItem();
        tree.deselectAll();
       
        for(int i = start+1; i < tree.getItemCount(); i++) {
            TreeItem ti = tree.getItem(i);
            if(ti.getText().toLowerCase().indexOf(name.toLowerCase()) >= 0) {
                tree.setSelection(ti);
                return true;
            }
        }
        
        return selectItemByName(name);
    }
    
    
    public boolean selectPrevItemByName(String name) {
        int start = getSelectedItem();
        tree.deselectAll();
        
        for(int i = start-1; i >= 0; i--) {
            TreeItem ti = tree.getItem(i);
            if(ti.getText().toLowerCase().indexOf(name.toLowerCase()) >= 0) {
                tree.setSelection(ti);
                return true;
            }
        }
        
        return selectItemByNameReverse(name);
    }
    
    
    private boolean triggerExpandTreeItem (DBConn db) {
        boolean r = true;
        
        for(int i=0; i<tree.getSelectionCount(); i++) {
            TreeItem ti = tree.getSelection()[i];
            Toolbar toolbar = Toolbar.getInstance(tree.getShell());
            toolbar.disableItems(Toolbar.ITEM_NEW | Toolbar.ITEM_OPEN);
            toolbar.removeItemListener(Toolbar.ITEM_OPEN);
            toolbar.removeItemListener(Toolbar.ITEM_NEW);
            
            try {
                r = setTreeItemExpandable(ti, db);
            } catch (Exception e) {
                logger.debug(e);
                return false;
            }
        }
        
        return r;
    }
    
    
    private boolean refreshTree (DBConn db) {
        boolean r = true;
        
        tree.deselectAll();
        Toolbar toolbar = Toolbar.getInstance(tree.getShell());
        toolbar.disableItems(Toolbar.ITEM_NEW | Toolbar.ITEM_OPEN);
        toolbar.removeItemListener(Toolbar.ITEM_OPEN);
        toolbar.removeItemListener(Toolbar.ITEM_NEW);        
        
        for(int i=0; i<tree.getItemCount(); i++) {
            TreeItem ti = tree.getItem(i);
            if(!ti.getExpanded()) continue;
            
            try {
                r = setTreeItemExpandable(ti, db);
            } catch (Exception e) {
                logger.debug(e);
                return false;
            }
        }
        
        return r;
    }
    
    
    public void fillTree(final DBConn db) {
        // TODO Check correctness of the result, etc...
        if(db == null) return;
        new FillTree(tree.getDisplay(), tree, db).start();
        
        tree.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event event) {
                triggerExpandTreeItem(db);
            }
        }
        );
        
        tree.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent event) { }
            
            @Override
            public void focusGained(FocusEvent event) {
                triggerExpandTreeItem(db);
            }
        });

        subTreeItemClickListener = new Listener() {

            public void handleEvent(Event event) {
                for(TreeItem sti : tree.getSelection()) {
                    if(CryptoEntry.class.isInstance(sti.getData()) && (event.keyCode == SWT.CR || event.keyCode == 0)) {
                        tfolders.addCryptoTab((CryptoEntry) sti.getData(), db, gui.getCrypto());
                    }
                    else if (sti.getItemCount() > 0) {
                    	sti.setExpanded(!sti.getExpanded());
                    	sti.setImage(sti.getExpanded() ? iconFolderOpen : iconFolder);
                    }
                }                
            }
        };
        tree.addListener(SWT.MouseDoubleClick, subTreeItemClickListener);
        tree.addListener(SWT.KeyDown, new Listener() {
            
            @Override
            public void handleEvent(Event event) {
                for(TreeItem sti : tree.getSelection()) {
                    if(sti.getItemCount() > 0) {
                    	if(event.keyCode == ' ' || event.keyCode == SWT.CR)
                    		sti.setExpanded(!sti.getExpanded());
                    	else if(event.keyCode == SWT.ARROW_RIGHT)
                    		sti.setExpanded(true);
                    	else if(event.keyCode == SWT.ARROW_LEFT)
                    		sti.setExpanded(false);

                    	if(sti.getExpanded())
                    		sti.setImage(iconFolderOpen);
                    	else
                    		sti.setImage(iconFolder);
                    }
                }
            }
        });
    }
    
    public void clearTree() {
        // TODO Check correctness of the result, etc...
        tree.removeAll();
    }
    
    
    private boolean setTreeItemExpandable(final TreeItem ti, final DBConn db) throws RoleNotFoundException, SRRoleKeyMismatchException, SRRoleNotFoundException {
    	try {
	        Object data = ti.getData();
	        final Image _iconLocked           = GUI.getIcon(tree.getDisplay(), "locked.png");
	        final Image _iconUnlocked         = GUI.getIcon(tree.getDisplay(), "unlocked.png");
            final Image _iconLockedReadOnly   = GUI.getIcon(tree.getDisplay(), "locked_readonly.png");
            final Image _iconUnlockedReadOnly = GUI.getIcon(tree.getDisplay(), "unlocked_readonly.png");
            final Image _iconLockedHidden     = GUI.getIcon(tree.getDisplay(), "locked_hidden.png");
            final Image _iconUnlockedHidden   = GUI.getIcon(tree.getDisplay(), "unlocked_hidden.png");            
	        
	        
	        if(Customer.class.isInstance(data)) {
	            boolean expanded = ti.getExpanded();
	            ti.removeAll();
	            
	            
	            Toolbar toolbar = Toolbar.getInstance(tree.getShell());
	            toolbar.enableItems(Toolbar.ITEM_NEW);
	            toolbar.removeItemListener(Toolbar.ITEM_NEW);
	            toolbar.addItemListener(Toolbar.ITEM_NEW, new Listener() {
                    
                    @Override
                    public void handleEvent(Event event) {
                        tfolders.addCryptoTab((Customer) ti.getData(), db, gui.getCrypto());                        
                    }
                });

	            Customer c = (Customer) data;

	            List<CryptoEntry> ces = db.getCryptoEntries(c);
	            if(ces == null) return false;
	            
	            for(Iterator<CryptoEntry> it = ces.iterator(); it.hasNext();) {
	                final CryptoEntry ce = it.next();
	                if(!mainmenu.getItemSelection(MainMenu.ITEM_SHOWHIDDEN) && ce.isHidden())
	                    continue;
	                TreeItem sti = new TreeItem(ti, SWT.NONE);
	
	                sti.setText(ce.getSubject());

	                List<UserRole> rolesByRole = db.getUserRolesByRole(ce.getRole());
	                boolean enable = false;
	                for(Iterator<UserRole> i = rolesByRole.iterator(); i.hasNext();) {
	                    UserRole userRoleByRole = i.next();
	                    
	                    if(userRoleByRole.getUserKeyFPR().equals(gui.getCrypto().getFPR(ECryptoPGP_KeyType.USER_KEY))) {
	                    	enable = true;
	                        break;
	                    }
	                }
	                if(ces.isEmpty() == false) ti.setImage(iconFolder);

                	sti.setForeground(sti.getDisplay().getSystemColor(enable ? SWT.COLOR_WIDGET_FOREGROUND : SWT.COLOR_TITLE_INACTIVE_FOREGROUND));
                	
                	Image iconUnlocked = _iconUnlocked;
                	Image iconLocked = _iconLocked;
                	if(ce.isHidden()) {
                	    iconUnlocked = _iconUnlockedHidden;
                	    iconLocked = _iconLockedHidden;
                	} else if (ce.isReadonly()) {
                	    iconUnlocked = _iconUnlockedReadOnly;
                	    iconLocked = _iconLockedReadOnly;
                	}
                	    
                	sti.setImage(enable ? iconUnlocked : iconLocked);
                	sti.setData(enable ? ce : null);
	            }

	            ti.setExpanded(expanded);
	        } else if(CryptoEntry.class.isInstance(data)) {
	            Toolbar toolbar = Toolbar.getInstance(tree.getShell());
	            
	            toolbar.enableItems(Toolbar.ITEM_OPEN);
	            toolbar.addItemListener(Toolbar.ITEM_OPEN, subTreeItemClickListener);
	        }
    	} catch (SWTException swte) {
    	    logger.debug(swte);
    		return false;
    	}
    	
    	return true;
    }
    
    
    class FillTree extends Thread {
        private Display display;
        private Tree    tree;
        private DBConn  db;
        
        public FillTree(Display display, Tree tree, DBConn db) {
            setName("FillTreeThread");
            this.display = display;
            this.tree    = tree;
            this.db      = db;
        }
        
        public void run() {
            display.asyncExec(new Runnable() {
                public void run() {
                    if(tree.isDisposed())
                        return;
                    
                    List<Customer> cust = db.getCustomers();
                    if(cust == null) return;
                    
                   	for(Iterator<Customer> i = cust.iterator(); i.hasNext();) {
                   		Customer c = i.next();
                            
                   		final TreeItem ti = new TreeItem(tree, SWT.NONE);
                   		c.setUserData(ti);
                   		ti.setData(c);

                        ti.setImage(iconFolderGrey);
                   		ti.setText(c.getName());
                   	}		
               	}
           	});
       	}
   	}
}
