/**
 * 
 */
package de.ibh.StrongRoom.Client.SWT;

import org.apache.log4j.Logger;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import de.ibh.StrongRoom.l10n.StrongRoomL10N;

/**
 * @author ellguth, lehmann
 *
 */
public class Toolbar {
    
    private ToolBar t = null;
    private static Toolbar toolbar = null; 
    private Map<Integer,ToolItem> _items = new HashMap<Integer, ToolItem>();
    private Map<Integer,MenuItem> _popupItems = new HashMap<Integer, MenuItem>();
    private final StrongRoomL10N  l10n = StrongRoomL10N.getInstance(); 
        
    static public final int ITEM_CLOSE      =     1;
    static public final int ITEM_NEW        =     2;
    static public final int ITEM_SAVE       =     4;
    static public final int ITEM_OPEN       =     8;
    static public final int ITEM_EXIT       =    16;
    static public final int ITEM_CONNECT    =    32;
    static public final int ITEM_DISCONNECT =    64;
    static public final int ITEM_BOLD       =   128;
    static public final int ITEM_ITALIC     =   256;
    static public final int ITEM_COLOR      =   512;
    static public final int ITEM_EDIT       =  1024;
    static public final int ITEM_UNDERLINE  =  2048;
    static public final int ITEM_BLACK      =  4096;
    static public final int ITEM_RED        =  8192;
    static public final int ITEM_GREEN      = 16384;
    static public final int ITEM_BLUE       = 32768;
    
    private ToolItem boldItem, italicItem, colorItem, underlineItem;
    private MenuItem blackItem, redItem, greenItem, blueItem; 
    
    static Logger logger = Logger.getLogger(GUI.class);
       
    
    public int getItemMask() {
        int mask = 0;
        
        Iterator<Map.Entry<Integer, ToolItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, ToolItem> item = items.next();
            if(((ToolItem) item.getValue()).getEnabled() == true)
                mask|=item.getKey();
        }
        
        return mask;
    }
    
    
    public void disableItems(int disitems) {
        Iterator<Map.Entry<Integer, ToolItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, ToolItem> item = items.next();
            if(((Integer) item.getKey() & disitems) == (Integer) item.getKey()) {
                ((ToolItem) item.getValue()).setEnabled(false);
            }
            
        }        
    }
    

    public void enableItems(int enaitems) {
        Iterator<Map.Entry<Integer, ToolItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, ToolItem> item = items.next();
            if(((Integer) item.getKey() & enaitems) == (Integer) item.getKey()) {
                ((ToolItem) item.getValue()).setEnabled(true);
            }
            
        }        
    }
    
    
    public void disableAllItems() {
        disableItems(0xFFFF);
    }
    
    
    public void enableAllItems() {
        enableItems(0xFFFF);
    }
    
    
    public HashMap<Integer, Listener []> getItemListeners() {
        HashMap<Integer, Listener []> hashmap = new HashMap<Integer, Listener[]>();
        
        for(Iterator<Integer> it = _items.keySet().iterator(); it.hasNext();) {
            Integer key = it.next();
            hashmap.put(key, _items.get(key).getListeners(SWT.Selection));
        }
        for(Iterator<Integer> it = _popupItems.keySet().iterator(); it.hasNext();) {
            Integer key = it.next();
            hashmap.put(key, _popupItems.get(key).getListeners(SWT.Selection));
        }        
        
        return hashmap;
    }
    
    
    public void setItemListeners(HashMap<Integer, Listener []> hashmap) {
        for(Iterator<Integer> it = hashmap.keySet().iterator(); it.hasNext();) {
            Integer key = it.next();
            removeItemListener(key);
            for(Listener listener : hashmap.get(key))
                addItemListener(key, listener);
        }
    }
    
    
    public void removeAllItemListeners() {
        for(Iterator<Integer> it = _items.keySet().iterator(); it.hasNext();)
            removeItemListener(it.next());
    }
    
        
    public void addItemListener(int item, Listener listener) {
        MenuItem _popupItem;
        if((_popupItem = _popupItems.get(item)) != null) {
            _popupItem.setData(item);
            _popupItem.addListener(SWT.Selection, listener);

            return;
        }
        
        ToolItem _item = _items.get(item);
        if(_item == null) return;
        
        _item.setData(item);
        _item.addListener(SWT.Selection, listener);
    }
    
    
    public void removeItemListener(int item) {
        MenuItem _popupItem;
        if((_popupItem = _popupItems.get(item)) != null) {
            for(Listener listener : _popupItem.getListeners(SWT.Selection))
                _popupItem.removeListener(SWT.Selection, listener);

            return;
        }        
        
        ToolItem _item = _items.get(item);
        if(_item == null) return;
        
        for(Listener listener : _item.getListeners(SWT.Selection))
            _item.removeListener(SWT.Selection, listener);
    }
    
    
    public void setBoldItemChecked(boolean enabled) {
        if(boldItem.getSelection() != enabled) boldItem.setSelection(enabled);
    }
    
    
    public void setItalicItemChecked(boolean enabled) {
        if(italicItem.getSelection() != enabled) italicItem.setSelection(enabled);
    }
    
    
    public void setUnderlineItemChecked(boolean enabled) {
        if(underlineItem.getSelection() != enabled) underlineItem.setSelection(enabled);
    }    
    
    
    public void setColorItemChecked(boolean enabled) {
        if(colorItem.getSelection() != enabled) colorItem.setSelection(enabled);
    }
    
    
    private void addTBItems(final Composite comp) {
    	if (t == null) return;
        
         
        // Connect button
        ToolItem _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Connect"));
        Image _icon = GUI.getIcon(comp.getDisplay(), "reconnect.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_CONNECT, _item);
        
        // Disconnect button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Disconnect"));
        _icon = GUI.getIcon(comp.getDisplay(), "disconnect.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_DISCONNECT, _item);
        
        
        // New button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("New"));
        _icon = GUI.getIcon(comp.getDisplay(), "new.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_NEW, _item);
        
        
        // Save button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText("Save");
        _icon = GUI.getIcon(comp.getDisplay(), "save.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_SAVE, _item);
        
        // Edit button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Edit"));
        _icon = GUI.getIcon(comp.getDisplay(), "edit.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_EDIT, _item);        
        
        
        // Open button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Open"));
        _icon = GUI.getIcon(comp.getDisplay(), "open.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_OPEN, _item);        
        
        
        // Close button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Close"));
        _icon = GUI.getIcon(comp.getDisplay(), "close.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_CLOSE, _item);
        
        
        // Exit button
        _item = new ToolItem(t, SWT.PUSH);
        _item.setToolTipText(l10n._("Exit"));
        _icon = GUI.getIcon(comp.getDisplay(), "exit.png");
        _item.setImage(_icon);
        
        _items.put(ITEM_EXIT, _item);
               
        // Separator button
        _item = new ToolItem(t, SWT.SEPARATOR);
        
        // Bold button
        boldItem = new ToolItem(t, SWT.CHECK);
        boldItem.setToolTipText(l10n._("Bold"));
        _icon = GUI.getIcon(comp.getDisplay(), "bold.png");
        boldItem.setImage(_icon);
        
        _items.put(ITEM_BOLD, boldItem);
     
        // Italic button
        italicItem = new ToolItem(t, SWT.CHECK);
        italicItem.setToolTipText(l10n._("Italic"));
        _icon = GUI.getIcon(comp.getDisplay(), "italic.png");
        italicItem.setImage(_icon);
        
        _items.put(ITEM_ITALIC, italicItem);
        
        // Underline button
        underlineItem = new ToolItem(t, SWT.CHECK);
        underlineItem.setToolTipText(l10n._("Underline"));
        _icon = GUI.getIcon(comp.getDisplay(), "underline.png");
        underlineItem.setImage(_icon);
        
        _items.put(ITEM_UNDERLINE, underlineItem);        
        
        // Color button
        colorItem = new ToolItem(t, SWT.PUSH);
        colorItem.setToolTipText(l10n._("Color"));
        _icon = GUI.getIcon(comp.getDisplay(), "color.png");
        colorItem.setImage(_icon);
        
        final Menu colorPopup = new Menu(comp.getShell(), SWT.POP_UP);
        
        blackItem = new MenuItem(colorPopup, SWT.PUSH);
        blackItem.setText(l10n._("Normal"));
        blackItem.setImage(GUI.getIcon(comp.getDisplay(), "black.png"));
        _popupItems.put(ITEM_BLACK, blackItem);
        
        redItem = new MenuItem(colorPopup, SWT.PUSH);
        redItem.setText(l10n._("Red"));
        redItem.setImage(GUI.getIcon(comp.getDisplay(), "red.png"));
        _popupItems.put(ITEM_RED, redItem);

        greenItem = new MenuItem(colorPopup, SWT.PUSH);
        greenItem.setText(l10n._("Green"));
        greenItem.setImage(GUI.getIcon(comp.getDisplay(), "green.png"));
        _popupItems.put(ITEM_GREEN, greenItem);
        
        blueItem = new MenuItem(colorPopup, SWT.PUSH);
        blueItem.setText(l10n._("Blue"));
        blueItem.setImage(GUI.getIcon(comp.getDisplay(), "blue.png"));
        _popupItems.put(ITEM_BLUE, blueItem);

        colorItem.addListener(SWT.Selection, new Listener() {
            @Override
            public void handleEvent(Event event) {
                colorPopup.setVisible(true);
            }
        });
        
        _items.put(ITEM_COLOR, colorItem);
    }
    
    public ToolItem getBoldItem(){
    	return boldItem;
    }
    
    public ToolItem getItalicItem(){
    	return italicItem;
    }
    
    public ToolItem getUnderlineItem() {
        return underlineItem;
    }
    
    public MenuItem getBlackItem() {
        return blackItem;
    }
    
    public MenuItem getRedItem() {
        return redItem;
    }
    
    public MenuItem getGreenItem() {
        return greenItem;
    }    
    
    public MenuItem getBlueItem() {
        return blueItem;
    }        
    
    private ToolBar addToolBar(Composite comp) {
        ToolBar _toolbar = new ToolBar(comp, SWT.BAR | SWT.NONE | SWT.FLAT);
        _toolbar.setLayoutData(new GridData(SWT.FILL, SWT.NONE, false, false, GUI.COLUMNS_MAX, 1));
        
        return _toolbar;
    }
    
    static public Toolbar getInstance(Composite parent) {
        if(toolbar != null)
            return toolbar;
        
        toolbar = new Toolbar(parent);
        
        return toolbar;
    }
    
    /**
     * @param parent
     */
    public Toolbar(Composite parent) {
        t = addToolBar(parent);
        
        addTBItems(parent);
    }   
}