package de.ibh.StrongRoom.DB;

import java.util.HashMap;
import java.util.Map;

import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;

public class CryptoEntry implements ISignedRow {
	private long id = -1;
	private Customer customer;
	private String subject;
	private String description;
	private Role role;
	private String rkey_fpr;
	private Boolean exportable;
	private Boolean readonly;
	private Boolean hidden;
	private String ukey_fpr;
	private String sign_rkey;
	private String sign_ukey;
	
	public CryptoEntry(final CryptoEntry ce) {
	    this.id = ce.getId();
	    this.customer = ce.getCustomer();
	    this.subject = ce.getSubject();
	    this.description = ce.getDescription();
	    this.role = ce.getRole();
	    this.rkey_fpr = ce.getRKey_FPR();
	    this.exportable = ce.isExportable();
	    this.readonly = ce.isReadonly();
	    this.hidden = ce.isHidden();
	    this.ukey_fpr = ce.getUKeyFPR();
	    this.sign_rkey = null;
	    this.sign_ukey = null;
	}
	
	public CryptoEntry(final long id, final Customer customer, final String subject, final String description, final Role role,
				final String rkey_fpr, final Boolean exportable, final Boolean readonly, final Boolean hidden,
				final String ukey_fpr, final String sign_rkey, final String sign_ukey) {
		this.id = id;
		this.customer = customer;
		this.subject = subject;
		this.description = description;
		this.role = role;
		this.rkey_fpr = rkey_fpr;
		this.exportable = exportable;
		this.readonly = readonly;
		this.hidden = hidden;
		this.ukey_fpr = ukey_fpr;
		this.sign_rkey = sign_rkey;
		this.sign_ukey = sign_ukey;
	}

	public long getId() {
		return id;
	}

	public Customer getCustomer() {
		return customer;
	}
	
	public String getSubject() {
		return subject;
	}
	
	public String getDescription() {
		return description;
	}

	public Role getRole() {
		return role;
	}

	public String getRKey_FPR() {
		return rkey_fpr;
	}

	public Boolean isExportable() {
		return exportable;
	}

	public Boolean isReadonly() {
		return readonly;
	}

	public Boolean isHidden() {
		return hidden;
	}

	public String getUKeyFPR() {
		return ukey_fpr;
	}
	
	@Override
	public String[] getSignStrings() {
		return new String[] {
				Long.toString(id),
				Long.toString(customer.getId()),
				subject,
				description,
				role.getRoleKey().getKeyFPR(),
				Boolean.toString(exportable),
				Boolean.toString(readonly),
				Boolean.toString(hidden),
				ukey_fpr,
		};
	}
	
	public void setReadonly(final boolean readonly) {
	    this.readonly = readonly;
	}
	
	public void setHidden(final boolean hidden) {
	    this.hidden = hidden;
	}
	
	public void setSubject(final String subject) {
	    this.subject = subject;
	}
	
	public void setDescription(final String description) {
	    this.description = description;
	}

    @Override
	public Map<ECryptoPGP_KeyType, String> getSignKeyMap() {
		Map<ECryptoPGP_KeyType, String> m = new HashMap<ECryptoPGP_KeyType, String>();

		m.put(ECryptoPGP_KeyType.ROLE_KEY, sign_rkey);
		m.put(ECryptoPGP_KeyType.USER_KEY, sign_ukey);

		return m;
	}

	@Override
	public void setSigningData(ECryptoPGP_KeyType kt, String sig) {
		if(kt == ECryptoPGP_KeyType.ROLE_KEY)
			sign_rkey = sig;
		else if(kt == ECryptoPGP_KeyType.USER_KEY)
			sign_ukey = sig;
		else
			throw new IllegalArgumentException();
	}
}
