package de.ibh.StrongRoom.DB;

import java.util.HashMap;
import java.util.Map;

import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;

public class Role implements ISignedRow {
	private long id;
	private RoleKey rKey;
	private String rKeyFPR;
	private String name;
	private String description;
	private boolean readonly;
	private boolean hidden;
	private String signSKey;

    Role(final long id, final RoleKey rKey, final String rKeyFPR, final String name, final String description, final boolean readonly, final boolean hidden, final String signSKey) {
        this.id = id;
        this.rKey = rKey;
        this.name = name;
        this.description = description;
        this.readonly = readonly;
        this.hidden = hidden;
        this.signSKey = signSKey;

        rKey.setRole(this);
    }
    
	public long getID() {
		return id;
	}

	public final RoleKey getRoleKey() {
		return rKey;
	}

	public String getName() {
		return name;
	}

	public String getDescription() {
		return description;
	}
	
	public boolean getReadonly() {
		return readonly;
	}
	
	public boolean getHidden() {
		return hidden;
	}
	
	@Override
	public String[] getSignStrings() {
		return new String[] {Long.toString(id), rKey.getKeyFPR(), rKeyFPR, name, description, Boolean.toString(readonly), Boolean.toString(hidden)};
	}

    @Override
	public Map<ECryptoPGP_KeyType, String> getSignKeyMap() {
		Map<ECryptoPGP_KeyType, String> m = new HashMap<ECryptoPGP_KeyType, String>();

		m.put(ECryptoPGP_KeyType.SIGNING_KEY, signSKey);

		return m;
	}

	@Override
	public void setSigningData(ECryptoPGP_KeyType kt, String sig) {
		if(kt == ECryptoPGP_KeyType.SIGNING_KEY)
			signSKey = sig;
		else
			throw new IllegalArgumentException();
	}
}
