//---------------------------------------------------------------------------
// Freely available from Media Art Online (http://www.media-art-online.org/).
// Copyright (C) 2003, 2004 Media Art Online (cafe@media-art-online.org)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// PROJECT: PGP/GnuPG Applications
// MODULE:  GnuPG support
// FILE:    GnuPGUser.java
//
// DESCRIPTION:
//
// REVISION HISTORY:
//  Name      Date        Description
//  ----      ----------  -----------
//  ks91      2003/01/11  initial version
//
// $Id: //depot/scm/main/java/org/media_art_online/gnupg/GnuPGUser.java#10 $
// $DateTime: 2006/07/24 17:39:58 $
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Package
//---------------------------------------------------------------------------
package org.media_art_online.gnupg;

//---------------------------------------------------------------------------
// Import
//---------------------------------------------------------------------------
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class GnuPGUser extends LinkedList<GnuPGName> {
	private static final long serialVersionUID = 1L;

	public GnuPGUser(String sValidity, String sTrust) {

		super();

		_keyPrimary = null;
		_loSubkeys  = null;
		_userPair   = null;

		_iTrust = (sTrust == null
		 ? GnuPG.TRUST_ULTIMATE : getIntValue(sTrust.charAt(0)));
		_iValidity   = (sValidity   == null
		 ? GnuPG.TRUST_ULTIMATE : getIntValue(sValidity.charAt(0)));

		_loSubkeys = new LinkedList<GnuPGKey>();

		_isSignatureChecked = false;
	}

	public void addName(GnuPGName name) {
		add(name);
	}

	public void addSubkey(GnuPGKey key) {
		_loSubkeys.add(key);
	}

	public boolean equals(GnuPGUser user) {
		return (getPrimaryKey().getFingerprint().equals(
		 user.getPrimaryKey().getFingerprint()));
	}

	public GnuPGPhoto getFirstPhoto() {

		GnuPGPhoto photo = null;

		for (Iterator<GnuPGName> iterator = iterator(); iterator.hasNext(); ) {

			GnuPGName name = iterator.next();

			if (name.getClass() == GnuPGPhoto.class) {
				photo = (GnuPGPhoto)name;
				break;
			}
		}

		return (photo);
	}

	/**
	 * This will only work after identifies() is called. The object must not
	 * be referred to from multiple threads.
	 */
	public GnuPGName getIdentifyingName() {
		return ((GnuPGName)get(_iIdentifying));
	}

	public GnuPGUser getPair() {
		return (_userPair);
	}

	public List<GnuPGPhoto> getPhotos() {

		List<GnuPGPhoto> list = new LinkedList<GnuPGPhoto>();

		for (Iterator<GnuPGName> iterator = iterator(); iterator.hasNext(); ) {

			GnuPGName name = iterator.next();

			if (name.getClass() == GnuPGPhoto.class) {
				list.add((GnuPGPhoto) name);
			}
		}

		return (list);
	}

	public GnuPGKey getPrimaryKey() {
		return (_keyPrimary);
	}

	public LinkedList<GnuPGKey> getSubkeys() {
		return (_loSubkeys);
	}

	public int getTrust() {
		return (_iTrust);
	}

	public int getValidity() {
		return (_iValidity);
	}

	public boolean identifies(GnuPGName name) {

		_iIdentifying = -1;

		for (int i = 0; i < size(); i++) {
			if (((GnuPGName)get(i)).equals(name)) {
				_iIdentifying = i;
				break;
			}
		}

		return (_iIdentifying >= 0);
	}

	public boolean identifies(GnuPGUser user) {

		boolean isIdentified = false;

		for (int i = 0; i < size(); i++) {
			if (user.identifies((GnuPGName)get(i))) {
				isIdentified = true;
				break;
			}
		}

		return (isIdentified);
	}

	public boolean isSignatureChecked() {
		return (_isSignatureChecked);
	}

	public boolean probablyEquals(GnuPGUser user) {
		return (getPrimaryKey().getID().equals(user.getPrimaryKey().getID()));
	}

	public void setPair(GnuPGUser user) {

		this._userPair = user;
		user._userPair = this;
	}

	public void setPrimaryKey(GnuPGKey key) {
		_keyPrimary = key;
	}

	public void setSignatureChecked(boolean isSignatureChecked) {
		_isSignatureChecked = isSignatureChecked;
	}

	public void setValidity(int iValidity) {
		_iValidity = iValidity;
	}

	public String toString() {
		return (((GnuPGName)get(0)).toString());
	}

//===========================================================================
// BARRIER: Anything below is not open to other files.
//===========================================================================

	private int getIntValue(char c) {

		int iValue = 0;

		switch (c) {
			case CASE_TRUST_INVALID:
				iValue = GnuPG.TRUST_INVALID;
				break;

			case CASE_TRUST_DISABLED:
				iValue = GnuPG.TRUST_DISABLED;
				break;

			case CASE_TRUST_REVOKED:
				iValue = GnuPG.TRUST_REVOKED;
				break;

			case CASE_TRUST_EXPIRED:
				iValue = GnuPG.TRUST_EXPIRED;
				break;

			case CASE_TRUST_NEVER:
				iValue = GnuPG.TRUST_NEVER;
				break;

			case CASE_TRUST_MARGINAL:
				iValue = GnuPG.TRUST_MARGINAL;
				break;

			case CASE_TRUST_FULL:
				iValue = GnuPG.TRUST_FULL;
				break;

			case CASE_TRUST_ULTIMATE:
				iValue = GnuPG.TRUST_ULTIMATE;
				break;

			case CASE_TRUST_UNKNOWN1:
			case CASE_TRUST_UNKNOWN2:
			case CASE_TRUST_UNDEFINED:
			default:
				iValue = GnuPG.TRUST_UNDEFINED;
				break;
		}

		return (iValue);
	}

	private GnuPGKey _keyPrimary;

	private GnuPGUser _userPair;

	private LinkedList<GnuPGKey> _loSubkeys;

	private int _iIdentifying;
	private int _iTrust;
	private int _iValidity;

	private boolean _isSignatureChecked;

	private static final char CASE_TRUST_UNKNOWN1  = 'o';
	private static final char CASE_TRUST_INVALID   = 'i';
	private static final char CASE_TRUST_DISABLED  = 'd';
	private static final char CASE_TRUST_REVOKED   = 'r';
	private static final char CASE_TRUST_EXPIRED   = 'e';
	private static final char CASE_TRUST_UNKNOWN2  = '-';
	private static final char CASE_TRUST_UNDEFINED = 'q';
	private static final char CASE_TRUST_NEVER     = 'n';
	private static final char CASE_TRUST_MARGINAL  = 'm';
	private static final char CASE_TRUST_FULL      = 'f';
	private static final char CASE_TRUST_ULTIMATE  = 'u';
};

// GnuPGUser.java
